@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -32,11 +32,11 @@ import java.io.InputStream;
  *
  * <p>The decompression requires large amounts of memory. Thus you
  * should call the {@link #close() close()} method as soon as
- * possible, to force <tt>CBZip2InputStream</tt> to release the
+ * possible, to force <code>CBZip2InputStream</code> to release the
  * allocated memory.  See {@link CBZip2OutputStream
  * CBZip2OutputStream} for information about memory usage.</p>
  *
- * <p><tt>CBZip2InputStream</tt> reads bytes from the compressed
+ * <p><code>CBZip2InputStream</code> reads bytes from the compressed
  * source stream via the single byte {@link java.io.InputStream#read()
  * read()} method exclusively. Thus you should consider to use a
  * buffered source stream.</p>
@@ -74,7 +74,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
 
     private int currentChar = -1;
 
-    private static final int EOF                  = 0;
+    private static final int EOF               = 0;
     private static final int START_BLOCK_STATE = 1;
     private static final int RAND_PART_A_STATE = 2;
     private static final int RAND_PART_B_STATE = 3;
@@ -85,8 +85,10 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
 
     private int currentState = START_BLOCK_STATE;
 
-    private int storedBlockCRC, storedCombinedCRC;
-    private int computedBlockCRC, computedCombinedCRC;
+    private int storedBlockCRC;
+    private int storedCombinedCRC;
+    private int computedBlockCRC;
+    private int computedCombinedCRC;
 
     // Variables used by setup* methods exclusively
 
@@ -112,7 +114,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
      * concatenated .bz2 files.
      *
      * <p>Although BZip2 headers are marked with the magic
-     * <tt>"Bz"</tt> this constructor expects the next byte in the
+     * <code>"Bz"</code> this constructor expects the next byte in the
      * stream to be the first one after the magic.  Thus callers have
      * to skip the first two bytes. Otherwise this constructor will
      * throw an exception. </p>
@@ -121,7 +123,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
      * @throws IOException
      *  if the stream content is malformed or an I/O error occurs.
      * @throws NullPointerException
-     *  if <tt>in == null</tt>
+     *  if <code>in == null</code>
      */
     public CBZip2InputStream(final InputStream in) throws IOException {
         this(in, false);
@@ -132,7 +134,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
      * read from the specified stream.
      *
      * <p>Although BZip2 headers are marked with the magic
-     * <tt>"Bz"</tt> this constructor expects the next byte in the
+     * <code>"Bz"</code> this constructor expects the next byte in the
      * stream to be the first one after the magic.  Thus callers have
      * to skip the first two bytes. Otherwise this constructor will
      * throw an exception. </p>
@@ -147,7 +149,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
      * @throws IOException
      *             if the stream content is malformed or an I/O error occurs.
      * @throws NullPointerException
-     *             if <tt>in == null</tt>
+     *             if <code>in == null</code>
      */
     public CBZip2InputStream(final InputStream in,
                              final boolean decompressConcatenated)
@@ -286,7 +288,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
         }
 
         int blockSize = this.in.read();
-        if ((blockSize < '1') || (blockSize > '9')) {
+        if (blockSize < '1' || blockSize > '9') {
             throw new IOException("Stream is not BZip2 formatted: illegal "
                                   + "blocksize " + (char) blockSize);
         }
@@ -359,7 +361,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
         }
     }
 
-    private void endBlock() throws IOException {
+    private void endBlock() {
         this.computedBlockCRC = this.crc.getFinalCRC();
 
         // A bad CRC is considered a fatal error.
@@ -660,7 +662,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
         int minLens_zt  = minLens[zt];
 
         while (nextSym != eob) {
-            if ((nextSym == RUNA) || (nextSym == RUNB)) {
+            if (nextSym == RUNA || nextSym == RUNB) {
                 int s = -1;
 
                 for (int n = 1; true; n <<= 1) {
@@ -853,7 +855,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
             tt[cftab[ll8[i] & 0xff]++] = i;
         }
 
-        if ((this.origPtr < 0) || (this.origPtr >= tt.length)) {
+        if (this.origPtr < 0 || this.origPtr >= tt.length) {
             throw new IOException("stream corrupted");
         }
 
@@ -983,7 +985,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
         }
     }
 
-    private static final class Data extends Object {
+    private static final class Data {
 
         // (with blockSize 900k)
         final boolean[] inUse   = new boolean[256];                                   //      256 byte
@@ -1037,7 +1039,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
             // it can happen, if the compressor mixed small and large
             // blocks.  Normally only the last block will be smaller
             // than others.
-            if ((ttShadow == null) || (ttShadow.length < length)) {
+            if (ttShadow == null || ttShadow.length < length) {
                 this.tt = ttShadow = new int[length];
             }
 
@@ -1046,7 +1048,7 @@ public class CBZip2InputStream extends InputStream implements BZip2Constants {
 
     }
 
-    private static void reportCRCError() throws IOException {
+    private static void reportCRCError() {
         // The clean way would be to throw an exception.
         //throw new IOException("crc error");
 
