@@ -57,59 +57,56 @@ package org.codehaus.plexus.util;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Vector;
 
 /**
- * Class for scanning a directory for files/directories which match certain criteria.
- * <p/>
- * These criteria consist of selectors and patterns which have been specified. With the selectors you can select which
+ * <p>Class for scanning a directory for files/directories which match certain criteria.</p>
+ * 
+ * <p>These criteria consist of selectors and patterns which have been specified. With the selectors you can select which
  * files you want to have included. Files which are not selected are excluded. With patterns you can include or exclude
- * files based on their filename.
- * <p/>
- * The idea is simple. A given directory is recursively scanned for all files and directories. Each file/directory is
+ * files based on their filename.</p>
+ * 
+ * <p>The idea is simple. A given directory is recursively scanned for all files and directories. Each file/directory is
  * matched against a set of selectors, including special support for matching against filenames with include and and
  * exclude patterns. Only files/directories which match at least one pattern of the include pattern list or other file
  * selector, and don't match any pattern of the exclude pattern list or fail to match against a required selector will
- * be placed in the list of files/directories found.
- * <p/>
- * When no list of include patterns is supplied, "**" will be used, which means that everything will be matched. When no
+ * be placed in the list of files/directories found.</p>
+ * 
+ * <p>When no list of include patterns is supplied, "**" will be used, which means that everything will be matched. When no
  * list of exclude patterns is supplied, an empty list is used, such that nothing will be excluded. When no selectors
- * are supplied, none are applied.
- * <p/>
- * The filename pattern matching is done as follows: The name to be matched is split up in path segments. A path segment
+ * are supplied, none are applied.</p>
+ * 
+ * <p>The filename pattern matching is done as follows: The name to be matched is split up in path segments. A path segment
  * is the name of a directory or file, which is bounded by <code>File.separator</code> ('/' under UNIX, '\' under
  * Windows). For example, "abc/def/ghi/xyz.java" is split up in the segments "abc", "def","ghi" and "xyz.java". The same
- * is done for the pattern against which should be matched.
- * <p/>
- * The segments of the name and the pattern are then matched against each other. When '**' is used for a path segment in
- * the pattern, it matches zero or more path segments of the name.
- * <p/>
- * There is a special case regarding the use of <code>File.separator</code>s at the beginning of the pattern and the
+ * is done for the pattern against which should be matched.</p>
+ * 
+ * <p>The segments of the name and the pattern are then matched against each other. When '**' is used for a path segment in
+ * the pattern, it matches zero or more path segments of the name.</p>
+ * 
+ * <p>There is a special case regarding the use of <code>File.separator</code>s at the beginning of the pattern and the
  * string to match:<br>
  * When a pattern starts with a <code>File.separator</code>, the string to match must also start with a
  * <code>File.separator</code>. When a pattern does not start with a <code>File.separator</code>, the string to match
- * may not start with a <code>File.separator</code>. When one of these rules is not obeyed, the string will not match.
- * <p/>
- * When a name path segment is matched against a pattern path segment, the following special characters can be used:<br>
- * '*' matches zero or more characters<br>
- * '?' matches one character.
- * <p/>
- * Examples:
- * <p/>
- * "**\*.class" matches all .class files/dirs in a directory tree.
- * <p/>
- * "test\a??.java" matches all files/dirs which start with an 'a', then two more characters and then ".java", in a
- * directory called test.
- * <p/>
- * "**" matches everything in a directory tree.
- * <p/>
- * "**\test\**\XYZ*" matches all files/dirs which start with "XYZ" and where there is a parent directory called test
- * (e.g. "abc\test\def\ghi\XYZ123").
- * <p/>
- * Case sensitivity may be turned off if necessary. By default, it is turned on.
- * <p/>
- * Example of usage:
+ * may not start with a <code>File.separator</code>. When one of these rules is not obeyed, the string will not match.</p>
  * 
+ * <p>When a name path segment is matched against a pattern path segment, the following special characters can be used:<br>
+ * '*' matches zero or more characters<br>
+ * '?' matches one character.</p>
+ * 
+ * Examples:
+ * <ul>
+ *   <li>"**\*.class" matches all .class files/dirs in a directory tree.</li>
+ *   <li>"test\a??.java" matches all files/dirs which start with an 'a', then two more characters and then ".java", in a
+ * directory called test.</li>
+ *   <li>"**" matches everything in a directory tree.</li>
+ *   <li>"**\test\**\XYZ*" matches all files/dirs which start with "XYZ" and where there is a parent directory called test
+ * (e.g. "abc\test\def\ghi\XYZ123").</li>
+ * </ul>
+ * 
+ * <p>Case sensitivity may be turned off if necessary. By default, it is turned on.</p>
+ * Example of usage:
  * <pre>
  * String[] includes = { "**\\*.class" };
  * String[] excludes = { "modules\\*\\**" };
@@ -121,14 +118,14 @@ import java.util.Vector;
  *
  * System.out.println( "FILES:" );
  * String[] files = ds.getIncludedFiles();
- * for ( int i = 0; i < files.length; i++ )
+ * for ( int i = 0; i &lt; files.length; i++ )
  * {
  *     System.out.println( files[i] );
  * }
  * </pre>
  * 
- * This will scan a directory called test for .class files, but excludes all files in all proper subdirectories of a
- * directory called "modules"
+ * <p>This will scan a directory called test for .class files, but excludes all files in all proper subdirectories of a
+ * directory called "modules"</p>
  *
  * @author Arnout J. Kuiper <a href="mailto:ajkuiper@wxs.nl">ajkuiper@wxs.nl</a>
  * @author Magesh Umasankar
@@ -324,11 +321,11 @@ public class DirectoryScanner
     }
 
     /**
-     * Top level invocation for a slow scan. A slow scan builds up a full list of excluded/included files/directories,
+     * <p>Top level invocation for a slow scan. A slow scan builds up a full list of excluded/included files/directories,
      * whereas a fast scan will only have full results for included files, as it ignores directories which can't
-     * possibly hold any included files/directories.
-     * <p/>
-     * Returns immediately if a slow scan has already been completed.
+     * possibly hold any included files/directories.</p>
+     * 
+     * <p>Returns immediately if a slow scan has already been completed.</p>
      */
     protected void slowScan()
     {
@@ -440,6 +437,11 @@ public class DirectoryScanner
             newfiles = noLinks.toArray( new String[noLinks.size()] );
         }
 
+        if ( filenameComparator != null )
+        {
+            Arrays.sort( newfiles, filenameComparator );
+        }
+
         for ( String newfile : newfiles )
         {
             String name = vpath + newfile;
@@ -586,14 +588,10 @@ public class DirectoryScanner
     }
 
     /**
-     * <p>
-     * Returns the names of the files which were selected out and therefore not ultimately included.
-     * </p>
-     * <p/>
-     * <p>
-     * The names are relative to the base directory. This involves performing a slow scan if one has not already been
-     * completed.
-     * </p>
+     * <p>Returns the names of the files which were selected out and therefore not ultimately included.</p>
+     * 
+     * <p>The names are relative to the base directory. This involves performing a slow scan if one has not already been
+     * completed.</p>
      *
      * @return the names of the files which were deselected.
      * @see #slowScan
@@ -653,14 +651,10 @@ public class DirectoryScanner
     }
 
     /**
-     * <p>
-     * Returns the names of the directories which were selected out and therefore not ultimately included.
-     * </p>
-     * <p/>
-     * <p>
-     * The names are relative to the base directory. This involves performing a slow scan if one has not already been
-     * completed.
-     * </p>
+     * <p>Returns the names of the directories which were selected out and therefore not ultimately included.</p>
+     * 
+     * <p>The names are relative to the base directory. This involves performing a slow scan if one has not already been
+     * completed.</p>
      *
      * @return the names of the directories which were deselected.
      * @see #slowScan
@@ -674,10 +668,9 @@ public class DirectoryScanner
     }
 
     /**
-     * Checks whether a given file is a symbolic link.
-     * <p/>
-     * <p>
-     * It doesn't really test for symbolic links but whether the canonical and absolute paths of the file are identical
+     * <p>Checks whether a given file is a symbolic link.</p>
+     * 
+     * <p>It doesn't really test for symbolic links but whether the canonical and absolute paths of the file are identical
      * - this may lead to false positives on some platforms.
      * </p>
      *
@@ -700,12 +693,10 @@ public class DirectoryScanner
     }
 
     /**
-     * Checks whether the parent of this file is a symbolic link.
-     * <p/>
-     * <p>
-     * For java versions prior to 7 It doesn't really test for symbolic links but whether the canonical and absolute
-     * paths of the file are identical - this may lead to false positives on some platforms.
-     * </p>
+     * <p>Checks whether the parent of this file is a symbolic link.</p>
+     * 
+     * <p>For java versions prior to 7 It doesn't really test for symbolic links but whether the canonical and absolute
+     * paths of the file are identical - this may lead to false positives on some platforms.</p>
      *
      * @param parent the parent directory of the file to test
      * @param name the name of the file to test.
