@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-2019 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,9 +15,10 @@
  */
 package io.micronaut.context.annotation;
 
+import jakarta.inject.Qualifier;
+
 import static java.lang.annotation.RetentionPolicy.RUNTIME;
 
-import javax.inject.Qualifier;
 import java.lang.annotation.Documented;
 import java.lang.annotation.Retention;
 
@@ -34,4 +35,8 @@ import java.lang.annotation.Retention;
 @Documented
 @Retention(RUNTIME)
 public @interface Primary {
+    /**
+     * The simple name of this annotation.
+     */
+    String SIMPLE_NAME = Primary.class.getSimpleName();
 }
