@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,9 +15,6 @@
  */
 package io.micronaut.core.annotation;
 
-import javax.annotation.Nonnull;
-import javax.annotation.Nullable;
-
 /**
  * An interface for types capable of resolving {@link AnnotationMetadata}.
  *
@@ -38,7 +35,7 @@ public interface AnnotationMetadataResolver {
      * @param type The type
      * @return The {@link AnnotationMetadata}
      */
-    default @Nonnull AnnotationMetadata resolveMetadata(@Nullable Class<?> type) {
+    default @NonNull AnnotationMetadata resolveMetadata(@Nullable Class<?> type) {
         return AnnotationMetadata.EMPTY_METADATA;
     }
 
@@ -48,7 +45,7 @@ public interface AnnotationMetadataResolver {
      * @param object The object
      * @return The {@link AnnotationMetadata}
      */
-    default @Nonnull AnnotationMetadata resolveMetadata(Object object) {
+    default @NonNull AnnotationMetadata resolveMetadata(Object object) {
         return resolveMetadata(object != null ? object.getClass() : null);
     }
 }
