@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-2019 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,7 +20,9 @@ import io.micronaut.context.BeanRegistration;
 import io.micronaut.context.annotation.*;
 import io.micronaut.context.exceptions.NoSuchBeanException;
 import io.micronaut.core.annotation.AnnotationMetadata;
+import io.micronaut.core.annotation.AnnotationUtil;
 import io.micronaut.core.annotation.Internal;
+import io.micronaut.core.annotation.NonNull;
 import io.micronaut.core.naming.NameResolver;
 import io.micronaut.core.util.ArgumentUtils;
 import io.micronaut.core.util.ArrayUtils;
@@ -32,12 +34,14 @@ import io.micronaut.inject.DisposableBeanDefinition;
 import io.micronaut.inject.ParametrizedBeanFactory;
 import io.micronaut.inject.qualifiers.Qualifiers;
 import io.micronaut.spring.context.aware.SpringAwareListener;
+import jakarta.inject.Singleton;
 import org.springframework.beans.BeansException;
 import org.springframework.beans.factory.*;
 import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
 import org.springframework.beans.factory.config.BeanDefinitionHolder;
 import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
 import org.springframework.beans.factory.config.DependencyDescriptor;
+import org.springframework.beans.factory.support.AbstractBeanFactory;
 import org.springframework.beans.factory.support.DefaultListableBeanFactory;
 import org.springframework.beans.factory.support.GenericBeanDefinition;
 import org.springframework.beans.factory.support.RootBeanDefinition;
@@ -45,11 +49,6 @@ import org.springframework.core.ResolvableType;
 import org.springframework.stereotype.Component;
 import org.springframework.util.ConcurrentReferenceHashMap;
 
-import javax.annotation.Nonnull;
-import javax.inject.Named;
-import javax.inject.Qualifier;
-import javax.inject.Scope;
-import javax.inject.Singleton;
 import java.io.Closeable;
 import java.lang.annotation.Annotation;
 import java.util.*;
@@ -66,9 +65,9 @@ import java.util.function.Supplier;
 public class MicronautBeanFactory extends DefaultListableBeanFactory implements ListableBeanFactory, AutowireCapableBeanFactory, HierarchicalBeanFactory, ConfigurableListableBeanFactory {
 
     private final BeanContext beanContext;
-    private final Map<String, BeanDefinitionReference<?>> beanDefinitionMap = new LinkedHashMap<>(200);
+    private final Map<String, BeanDefinition<?>> beanDefinitionMap = new LinkedHashMap<>(200);
     // only used for by name lookups
-    private final Map<String, BeanDefinitionReference<?>> beanDefinitionsByName = new LinkedHashMap<>(200);
+    private final Map<String, BeanDefinition<?>> beanDefinitionsByName = new LinkedHashMap<>(200);
     private final SpringAwareListener springAwareListener;
     private final Map<String, Optional<Class<?>>> beanTypeCache = new ConcurrentReferenceHashMap<>();
     private final Map<Class, String[]> beanNamesForTypeCache = new ConcurrentReferenceHashMap<>();
@@ -107,25 +106,25 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
                     Collection<? extends BeanDefinition<?>> beanDefinitions = beanContext.getBeanDefinitions(definition.getBeanType());
                     for (BeanDefinition<?> beanDefinition : beanDefinitions) {
                         String beanName = computeBeanName(beanDefinition);
-                        beanDefinitionMap.put(beanName, reference);
+                        beanDefinitionMap.put(beanName, beanDefinition);
                     }
                 } else {
                     String beanName = computeBeanName(definition);
-                    beanDefinitionMap.put(beanName, reference);
+                    beanDefinitionMap.put(beanName, definition);
                 }
 
                 // handle component differently so that the value is a unique bean name
                 if (definition.isAnnotationPresent(Component.class)) {
                     // explicit handling of named beans
                     final Optional<String> v = definition.getValue(Component.class, String.class);
-                    v.ifPresent(s -> beanDefinitionsByName.put(s, reference));
+                    v.ifPresent(s -> beanDefinitionsByName.put(s, definition));
                 }
 
                 // handle Spring's @Bean differently so that the value is a unique bean name
                 if (definition.isAnnotationPresent(org.springframework.context.annotation.Bean.class)) {
                     // explicit handling of named beans
                     final Optional<String> v = definition.getValue(org.springframework.context.annotation.Bean.class, String.class);
-                    v.ifPresent(s -> beanDefinitionsByName.put(s, reference));
+                    v.ifPresent(s -> beanDefinitionsByName.put(s, definition));
                 }
             }
         }
@@ -137,12 +136,22 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
      * @return True if is singleton
      */
     public static boolean isSingleton(AnnotationMetadata annotationMetadata) {
-        if (annotationMetadata.isAnnotationPresent(EachProperty.class) || annotationMetadata.isAnnotationPresent(EachBean.class)) {
+        if (annotationMetadata.hasDeclaredStereotype(org.springframework.context.annotation.Bean.class)) {
+            final String scope = annotationMetadata.stringValue(org.springframework.context.annotation.Scope.class).orElse(null);
+            return scope == null || "singleton".equalsIgnoreCase(scope);
+        }
+        if (annotationMetadata.hasDeclaredStereotype(AnnotationUtil.SINGLETON)) {
             return true;
+        } else {
+            if (!annotationMetadata.hasDeclaredStereotype(AnnotationUtil.SCOPE) &&
+                    annotationMetadata.hasDeclaredStereotype(DefaultScope.class)) {
+                return annotationMetadata.stringValue(DefaultScope.class)
+                        .map(t -> t.equals(Singleton.class.getName()) || t.equals(AnnotationUtil.SINGLETON))
+                        .orElse(false);
+            } else {
+                return false;
+            }
         }
-        final Optional<Class<? extends Annotation>> scope = annotationMetadata.getDeclaredAnnotationTypeByStereotype(Scope.class);
-        // is singleton logic
-        return (scope.isPresent() && scope.get() == Singleton.class) || annotationMetadata.getValue(DefaultScope.class, Singleton.class).isPresent();
     }
 
     private String computeBeanName(BeanDefinition<?> definition) {
@@ -150,16 +159,16 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
         if (definition instanceof NameResolver) {
             name = ((NameResolver) definition).resolveName().orElse(Primary.class.getSimpleName());
         } else {
-            name = definition.getValue(Named.class, String.class).orElseGet(() ->
-                    definition.getAnnotationTypeByStereotype(Qualifier.class).map(Class::getSimpleName).orElse(definition.getClass().getSimpleName())
+            name = definition.getValue(AnnotationUtil.NAMED, String.class).orElseGet(() ->
+                    definition.getAnnotationTypeByStereotype(AnnotationUtil.QUALIFIER).map(Class::getSimpleName).orElse(definition.getClass().getSimpleName())
             );
         }
         return definition.getBeanType().getName() + "(" + name + ")";
     }
 
     @Override
-    public @Nonnull
-    Object getBean(@Nonnull String name) throws BeansException {
+    public @NonNull
+    Object getBean(@NonNull String name) throws BeansException {
         if (super.isAlias(name)) {
             final String[] aliases = super.getAliases(name);
             for (String alias : aliases) {
@@ -176,23 +185,35 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
             return singleton;
         } else {
             Class<?> type = getType(name);
-            BeanDefinitionReference<?> reference;
+            BeanDefinition<?> definition;
             if (type != null) {
-                reference = beanDefinitionMap.get(name);
+                definition = beanDefinitionMap.get(name);
             } else {
-                reference = beanDefinitionsByName.get(name);
-                if (reference != null) {
-                    type = reference.getBeanType();
+                definition = beanDefinitionsByName.get(name);
+                if (definition != null) {
+                    type = definition.getBeanType();
                 }
             }
 
-            if (reference != null && type != null) {
+            if (definition != null && type != null) {
 
-                AnnotationMetadata annotationMetadata = reference.getAnnotationMetadata();
-                Optional<Class<? extends Annotation>> q = annotationMetadata.getAnnotationTypeByStereotype(Qualifier.class);
+                AnnotationMetadata annotationMetadata = definition.getAnnotationMetadata();
+                Optional<io.micronaut.context.Qualifier> q = annotationMetadata.getAnnotationNameByStereotype(AnnotationUtil.QUALIFIER)
+                        .map(clazz -> Qualifiers.byAnnotation(annotationMetadata, clazz));
+
+                if (!q.isPresent()) {
+                    if (definition instanceof NameResolver) {
+                        name = ((NameResolver) definition).resolveName().orElse(null);
+                    } else {
+                        name = definition.stringValue(AnnotationUtil.NAMED).orElse(null);
+                    }
+                    if (name != null) {
+                        q = Optional.of(Qualifiers.byName(name));
+                    }
+                }
                 if (q.isPresent()) {
                     try {
-                        return beanContext.getBean(type, Qualifiers.byAnnotation(annotationMetadata, q.get()));
+                        return beanContext.getBean(type, q.get());
                     } catch (NoSuchBeanException e) {
                         throw new NoSuchBeanDefinitionException(type, e.getMessage());
                     } catch (Exception e) {
@@ -207,8 +228,8 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> T getBean(@Nonnull String name, @Nonnull Class<T> requiredType) throws BeansException {
+    public @NonNull
+    <T> T getBean(@NonNull String name, @NonNull Class<T> requiredType) throws BeansException {
         ArgumentUtils.requireNonNull("requiredType", requiredType);
         if (beanExcludes.contains(requiredType)) {
             throw new NoSuchBeanDefinitionException(requiredType);
@@ -238,8 +259,8 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    Object getBean(@Nonnull String name, @Nonnull Object... args) throws BeansException {
+    public @NonNull
+    Object getBean(@NonNull String name, @NonNull Object... args) throws BeansException {
         final Class<?> type = getType(name);
         if (type != null) {
             return beanContext.createBean(type, args);
@@ -248,8 +269,8 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> T getBean(@Nonnull Class<T> requiredType) throws BeansException {
+    public @NonNull
+    <T> T getBean(@NonNull Class<T> requiredType) throws BeansException {
         ArgumentUtils.requireNonNull("requiredType", requiredType);
         if (beanExcludes.contains(requiredType)) {
             throw new NoSuchBeanDefinitionException(requiredType);
@@ -268,8 +289,8 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> T getBean(@Nonnull Class<T> requiredType, @Nonnull Object... args) throws BeansException {
+    public @NonNull
+    <T> T getBean(@NonNull Class<T> requiredType, @NonNull Object... args) throws BeansException {
         try {
             return beanContext.createBean(requiredType, args);
         } catch (NoSuchBeanException e) {
@@ -278,8 +299,8 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> ObjectProvider<T> getBeanProvider(@Nonnull Class<T> requiredType) {
+    public @NonNull
+    <T> ObjectProvider<T> getBeanProvider(@NonNull Class<T> requiredType) {
         return new ObjectProvider<T>() {
             @Override
             public T getObject(Object... args) throws BeansException {
@@ -311,14 +332,14 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> ObjectProvider<T> getBeanProvider(@Nonnull ResolvableType requiredType) {
+    public @NonNull
+    <T> ObjectProvider<T> getBeanProvider(@NonNull ResolvableType requiredType) {
         final Class<T> resolved = (Class<T>) requiredType.resolve();
         return getBeanProvider(resolved);
     }
 
     @Override
-    public boolean containsBean(@Nonnull String name) {
+    public boolean containsBean(@NonNull String name) {
         return super.containsSingleton(name) ||
                 beanDefinitionMap.containsKey(name) ||
                 beanDefinitionsByName.containsKey(name) ||
@@ -326,11 +347,11 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public boolean isSingleton(@Nonnull String name) throws NoSuchBeanDefinitionException {
+    public boolean isSingleton(@NonNull String name) throws NoSuchBeanDefinitionException {
         if (super.containsSingleton(name)) {
             return true;
         } else {
-            final BeanDefinitionReference<?> definition = beanDefinitionMap.get(name);
+            final BeanDefinition<?> definition = beanDefinitionMap.get(name);
             if (definition != null) {
                 return isSingleton(definition);
             }
@@ -343,24 +364,24 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
      * @param definition The definition
      * @return True if it is
      */
-    protected boolean isSingleton(@Nonnull BeanDefinitionReference<?> definition) {
+    protected boolean isSingleton(@NonNull BeanDefinitionReference<?> definition) {
         final AnnotationMetadata annotationMetadata = definition.getAnnotationMetadata();
         return isSingleton(annotationMetadata);
     }
 
     @Override
-    public boolean isPrototype(@Nonnull String name) throws NoSuchBeanDefinitionException {
+    public boolean isPrototype(@NonNull String name) throws NoSuchBeanDefinitionException {
         if (super.containsSingleton(name)) {
             return false;
         }
 
-        final BeanDefinitionReference<?> definition = beanDefinitionMap.get(name);
+        final BeanDefinition<?> definition = beanDefinitionMap.get(name);
         if (definition != null) {
             final AnnotationMetadata annotationMetadata = definition.getAnnotationMetadata();
             if (annotationMetadata.hasDeclaredStereotype(Prototype.class)) {
                 return true;
             } else {
-                final boolean hasScope = annotationMetadata.getAnnotationNamesByStereotype(Scope.class).isEmpty();
+                final boolean hasScope = annotationMetadata.getAnnotationNamesByStereotype(AnnotationUtil.SCOPE).isEmpty();
                 return !hasScope;
             }
         }
@@ -368,7 +389,7 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public boolean isTypeMatch(@Nonnull String name, @Nonnull ResolvableType typeToMatch) throws NoSuchBeanDefinitionException {
+    public boolean isTypeMatch(@NonNull String name, @NonNull ResolvableType typeToMatch) throws NoSuchBeanDefinitionException {
         final Class<?> resolved = typeToMatch.resolve();
         if (resolved != null) {
             return isTypeMatch(name, resolved);
@@ -377,7 +398,7 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public boolean isTypeMatch(@Nonnull String name, @Nonnull Class<?> typeToMatch) throws NoSuchBeanDefinitionException {
+    public boolean isTypeMatch(@NonNull String name, @NonNull Class<?> typeToMatch) throws NoSuchBeanDefinitionException {
         final Class<?> beanType = getType(name);
         if (beanType != null) {
             return typeToMatch.isAssignableFrom(beanType);
@@ -386,11 +407,11 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public Class<?> getType(@Nonnull String beanName) throws NoSuchBeanDefinitionException {
+    public Class<?> getType(@NonNull String beanName) throws NoSuchBeanDefinitionException {
         Optional<Class<?>> opt = beanTypeCache.get(beanName);
         //noinspection OptionalAssignedToNull
         if (opt == null) {
-            final BeanDefinitionReference<?> definition = beanDefinitionMap.get(beanName);
+            final BeanDefinition<?> definition = beanDefinitionMap.get(beanName);
             if (definition != null) {
                 opt = Optional.of(definition.getBeanType());
             } else {
@@ -456,7 +477,7 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public boolean containsBeanDefinition(@Nonnull String beanName) {
+    public boolean containsBeanDefinition(@NonNull String beanName) {
         return beanDefinitionMap.containsKey(beanName);
     }
 
@@ -466,14 +487,14 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
+    public @NonNull
     String[] getBeanDefinitionNames() {
         return beanDefinitionMap.keySet().toArray(new String[0]);
     }
 
     @Override
-    public @Nonnull
-    String[] getBeanNamesForType(@Nonnull ResolvableType type) {
+    public @NonNull
+    String[] getBeanNamesForType(@NonNull ResolvableType type) {
         final Class<?> resolved = type.resolve();
         if (resolved != null) {
             return getBeanNamesForType(resolved);
@@ -482,13 +503,13 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
+    public @NonNull
     String[] getBeanNamesForType(Class<?> type) {
         return getBeanNamesForType(type, true, true);
     }
 
     @Override
-    public @Nonnull
+    public @NonNull
     String[] getBeanNamesForType(Class<?> type, boolean includeNonSingletons, boolean allowEagerInit) {
         // ignore certain common cases
         if (type == null || Object.class == type || List.class == type || beanExcludes.contains(type)) {
@@ -509,7 +530,7 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
+    public @NonNull
     <T> Map<String, T> getBeansOfType(Class<T> type, boolean includeNonSingletons, boolean allowEagerInit) throws BeansException {
         if (type == null || beanExcludes.contains(type)) {
             return Collections.emptyMap();
@@ -529,19 +550,19 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    String[] getBeanNamesForAnnotation(@Nonnull Class<? extends Annotation> annotationType) {
+    public @NonNull
+    String[] getBeanNamesForAnnotation(@NonNull Class<? extends Annotation> annotationType) {
         final String[] beanNamesForAnnotation = super.getBeanNamesForAnnotation(annotationType);
         final Collection<BeanDefinition<?>> beanDefinitions = beanContext.getBeanDefinitions(Qualifiers.byStereotype(annotationType));
         return ArrayUtils.concat(beansToNames(beanDefinitions), beanNamesForAnnotation);
     }
 
     @Override
-    public <A extends Annotation> A findAnnotationOnBean(@Nonnull String beanName, @Nonnull Class<A> annotationType) throws NoSuchBeanDefinitionException {
+    public <A extends Annotation> A findAnnotationOnBean(@NonNull String beanName, @NonNull Class<A> annotationType) throws NoSuchBeanDefinitionException {
         if (super.containsSingleton(beanName)) {
             return super.findAnnotationOnBean(beanName, annotationType);
         } else {
-            final BeanDefinitionReference<?> ref = beanDefinitionMap.get(beanName);
+            final BeanDefinition<?> ref = beanDefinitionMap.get(beanName);
             if (ref != null) {
                 return ref.getAnnotationMetadata().synthesize(annotationType);
             }
@@ -556,14 +577,14 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
     }
 
     @Override
-    public @Nonnull
-    <T> T createBean(@Nonnull Class<T> beanClass) throws BeansException {
+    public @NonNull
+    <T> T createBean(@NonNull Class<T> beanClass) throws BeansException {
         return beanContext.createBean(beanClass);
     }
 
     @SuppressWarnings("ConstantConditions")
     @Override
-    public void autowireBean(@Nonnull Object existingBean) throws BeansException {
+    public void autowireBean(@NonNull Object existingBean) throws BeansException {
         if (existingBean != null) {
             beanContext.inject(existingBean);
         }
@@ -648,14 +669,11 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
 
     @Override
     public org.springframework.beans.factory.config.BeanDefinition getBeanDefinition(String beanName) throws NoSuchBeanDefinitionException {
-        final BeanDefinitionReference<?> reference = beanDefinitionMap.get(beanName);
-        if (reference != null) {
-            final BeanDefinition<?> def = reference.load(beanContext);
-            if (def.isEnabled(beanContext)) {
-                final GenericBeanDefinition genericBeanDefinition = new GenericBeanDefinition();
-                genericBeanDefinition.setBeanClass(def.getBeanType());
-                return genericBeanDefinition;
-            }
+        final BeanDefinition<?> definition = beanDefinitionMap.get(beanName);
+        if (definition != null && definition.isEnabled(beanContext)) {
+            final GenericBeanDefinition genericBeanDefinition = new GenericBeanDefinition();
+            genericBeanDefinition.setBeanClass(definition.getBeanType());
+            return genericBeanDefinition;
         }
         throw new NoSuchBeanDefinitionException(beanName);
     }
@@ -675,51 +693,67 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
             }
         }
 
-        BeanDefinitionReference<?> reference = beanDefinitionMap.get(name);
-        if (reference == null) {
+        BeanDefinition<?> definition = beanDefinitionMap.get(name);
+        if (definition == null) {
             // by name, with no type lookups
-            final BeanDefinitionReference<?> ref = beanDefinitionsByName.get(name);
-            if (ref != null) {
+            final BeanDefinition<?> def = beanDefinitionsByName.get(name);
+            if (def != null) {
                 if (requiredType != null) {
-                    if (requiredType.isAssignableFrom(ref.getBeanType())) {
-                        reference = ref;
+                    if (requiredType.isAssignableFrom(def.getBeanType())) {
+                        definition = def;
                     }
                 } else {
-                    reference = ref;
+                    definition = def;
                 }
             }
         }
 
-        if (reference != null) {
-            final BeanDefinition<?> definition = reference.load(beanContext);
-            if (definition.isEnabled(beanContext)) {
-                if (requiredType == null) {
-                    requiredType = (Class<T>) definition.getBeanType();
-                }
+        if (definition != null && definition.isEnabled(beanContext)) {
+            if (requiredType == null) {
+                requiredType = (Class<T>) definition.getBeanType();
+            }
 
-                io.micronaut.context.Qualifier<T> q = (io.micronaut.context.Qualifier<T>) definition.getValue(Named.class, String.class)
-                        .map((String n) -> {
-                            if (Primary.class.getName().equals(n)) {
-                                return n;
-                            }
-                            return Qualifiers.byName(n);
-                        })
-                        .orElseGet(() -> {
-                                    if (definition.hasDeclaredStereotype(Primary.class)) {
-                                        return null;
-                                    }
-                                    final Class annotationType = definition.getAnnotationTypeByStereotype(Qualifier.class).orElse(null);
-                                    if (annotationType != null) {
-                                        return Qualifiers.byAnnotation(definition, annotationType);
-                                    }
+            BeanDefinition<?> finalDefinition = definition;
+            io.micronaut.context.Qualifier<T> q = (io.micronaut.context.Qualifier<T>) definition.getValue(AnnotationUtil.NAMED, String.class)
+                    .map((String n) -> {
+                        if (Primary.class.getName().equals(n)) {
+                            return n;
+                        }
+                        return Qualifiers.byName(n);
+                    })
+                    .orElseGet(() -> {
+                                if (finalDefinition.hasDeclaredStereotype(Primary.class)) {
                                     return null;
                                 }
-                        );
-                if (q != null) {
-                    return beanContext.getBean(requiredType, q);
-                } else {
-                    return beanContext.getBean(requiredType);
-                }
+                                final String annotationType = finalDefinition.getAnnotationNameByStereotype(AnnotationUtil.QUALIFIER).orElse(null);
+                                if (annotationType != null) {
+                                    return Qualifiers.byAnnotation(finalDefinition, annotationType);
+                                } else if (finalDefinition instanceof NameResolver) {
+                                    return ((NameResolver) finalDefinition).resolveName().map(Qualifiers::byName).orElse(null);
+                                }
+                        return null;
+                            }
+                    );
+            if (q != null) {
+                return beanContext.getBean(requiredType, q);
+            } else {
+                return beanContext.getBean(requiredType);
+            }
+        }
+
+        final org.springframework.beans.factory.BeanFactory parentBeanFactory = getParentBeanFactory();
+        if (parentBeanFactory != null) {
+            if (parentBeanFactory instanceof AbstractBeanFactory) {
+                return ((AbstractBeanFactory) parentBeanFactory).getBean(
+                        name, requiredType, args);
+            } else if (args != null) {
+                // Delegation to parent with explicit args.
+                return (T) parentBeanFactory.getBean(name, args);
+            } else if (requiredType != null) {
+                // No args -> delegate to standard getBean method.
+                return parentBeanFactory.getBean(name, requiredType);
+            } else {
+                return (T) parentBeanFactory.getBean(name);
             }
         }
 
@@ -764,12 +798,9 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
 
     @Override
     public void destroyBean(String beanName, Object beanInstance) {
-        final BeanDefinitionReference<?> reference = beanDefinitionMap.get(beanName);
-        if (reference != null) {
-            final BeanDefinition<?> ref = reference.load(beanContext);
-            if (ref instanceof DisposableBeanDefinition) {
-                ((DisposableBeanDefinition) ref).dispose(beanContext, beanInstance);
-            }
+        final BeanDefinition<?> definition = beanDefinitionMap.get(beanName);
+        if (definition instanceof DisposableBeanDefinition) {
+            ((DisposableBeanDefinition) definition).dispose(beanContext, beanInstance);
         }
     }
 
@@ -802,14 +833,13 @@ public class MicronautBeanFactory extends DefaultListableBeanFactory implements
 
     @Override
     protected boolean isPrimary(String beanName, Object beanInstance) {
-        BeanDefinitionReference<?> reference = beanDefinitionMap.get(beanName);
-        if (reference == null) {
-            reference = beanDefinitionsByName.get(beanName);
+        BeanDefinition<?> definition = beanDefinitionMap.get(beanName);
+        if (definition == null) {
+            definition = beanDefinitionsByName.get(beanName);
         }
 
-        if (reference != null) {
-            final BeanDefinition<?> definition = reference.load(beanContext);
-            return definition.hasDeclaredStereotype(Primary.class) || definition.getValue(Named.class, String.class).map((String n) -> Primary.class.getName().equals(n)).orElse(false);
+        if (definition != null) {
+            return definition.hasDeclaredStereotype(Primary.class) || definition.getValue(AnnotationUtil.NAMED, String.class).map((String n) -> Primary.class.getName().equals(n)).orElse(false);
         } else {
             return false;
         }
