@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2017 the original author or authors.
+ * Copyright 2012-2021 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ package org.springframework.boot.loader.tools;
  * custom loader classes.
  *
  * @author Phillip Webb
+ * @since 1.0.0
  * @see Layouts
  * @see RepackagingLayout
  * @see CustomLoaderLayout
@@ -38,10 +39,10 @@ public interface Layout {
 	 * Returns the destination path for a given library.
 	 * @param libraryName the name of the library (excluding any path)
 	 * @param scope the scope of the library
-	 * @return the destination relative to the root of the archive (should end with '/')
-	 * or {@code null} if the library should not be included.
+	 * @return the location of the library relative to the root of the archive (should end
+	 * with '/') or {@code null} if the library should not be included.
 	 */
-	String getLibraryDestination(String libraryName, LibraryScope scope);
+	String getLibraryLocation(String libraryName, LibraryScope scope);
 
 	/**
 	 * Returns the location of classes within the archive.
@@ -49,6 +50,28 @@ public interface Layout {
 	 */
 	String getClassesLocation();
 
+	/**
+	 * Returns the location of the classpath index file that should be written or
+	 * {@code null} if not index is required. The result should include the filename and
+	 * is relative to the root of the jar.
+	 * @return the classpath index file location
+	 * @since 2.5.0
+	 */
+	default String getClasspathIndexFileLocation() {
+		return null;
+	}
+
+	/**
+	 * Returns the location of the layer index file that should be written or {@code null}
+	 * if not index is required. The result should include the filename and is relative to
+	 * the root of the jar.
+	 * @return the layer index file location
+	 * @since 2.5.0
+	 */
+	default String getLayersIndexFileLocation() {
+		return null;
+	}
+
 	/**
 	 * Returns if loader classes should be included to make the archive executable.
 	 * @return if the layout is executable
