@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2019 the original author or authors.
+ * Copyright 2002-2020 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,7 +27,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.NoSuchElementException;
 import java.util.Optional;
-import java.util.function.BiFunction;
 import java.util.function.Function;
 import java.util.function.Predicate;
 
@@ -64,6 +63,7 @@ import org.springframework.util.ReflectionUtils;
  *
  * @author Phillip Webb
  * @author Juergen Hoeller
+ * @author Sam Brannen
  * @since 5.2
  * @param <A> the annotation type
  * @see TypeMappedAnnotations
@@ -73,18 +73,19 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	private static final Map<Class<?>, Object> EMPTY_ARRAYS;
 	static {
 		Map<Class<?>, Object> emptyArrays = new HashMap<>();
-		emptyArrays.put(String.class, new String[] {});
-		emptyArrays.put(boolean.class, new boolean[] {});
-		emptyArrays.put(byte.class, new byte[] {});
-		emptyArrays.put(char.class, new char[] {});
-		emptyArrays.put(double.class, new double[] {});
-		emptyArrays.put(float.class, new float[] {});
-		emptyArrays.put(int.class, new int[] {});
-		emptyArrays.put(long.class, new long[] {});
-		emptyArrays.put(short.class, new short[] {});
+		emptyArrays.put(boolean.class, new boolean[0]);
+		emptyArrays.put(byte.class, new byte[0]);
+		emptyArrays.put(char.class, new char[0]);
+		emptyArrays.put(double.class, new double[0]);
+		emptyArrays.put(float.class, new float[0]);
+		emptyArrays.put(int.class, new int[0]);
+		emptyArrays.put(long.class, new long[0]);
+		emptyArrays.put(short.class, new short[0]);
+		emptyArrays.put(String.class, new String[0]);
 		EMPTY_ARRAYS = Collections.unmodifiableMap(emptyArrays);
 	}
 
+
 	private final AnnotationTypeMapping mapping;
 
 	@Nullable
@@ -96,7 +97,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	@Nullable
 	private final Object rootAttributes;
 
-	private final BiFunction<Method, Object, Object> valueExtractor;
+	private final ValueExtractor valueExtractor;
 
 	private final int aggregateIndex;
 
@@ -109,21 +110,17 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 
 	private final int[] resolvedMirrors;
 
-	@Nullable
-	private String string;
-
 
 	private TypeMappedAnnotation(AnnotationTypeMapping mapping, @Nullable ClassLoader classLoader,
-			@Nullable Object source, @Nullable Object rootAttributes,
-			BiFunction<Method, Object, Object> valueExtractor, int aggregateIndex) {
+			@Nullable Object source, @Nullable Object rootAttributes, ValueExtractor valueExtractor,
+			int aggregateIndex) {
 
 		this(mapping, classLoader, source, rootAttributes, valueExtractor, aggregateIndex, null);
 	}
 
 	private TypeMappedAnnotation(AnnotationTypeMapping mapping, @Nullable ClassLoader classLoader,
-			@Nullable Object source, @Nullable Object rootAttributes,
-			BiFunction<Method, Object, Object> valueExtractor, int aggregateIndex,
-			@Nullable int[] resolvedRootMirrors) {
+			@Nullable Object source, @Nullable Object rootAttributes, ValueExtractor valueExtractor,
+			int aggregateIndex, @Nullable int[] resolvedRootMirrors) {
 
 		this.mapping = mapping;
 		this.classLoader = classLoader;
@@ -140,9 +137,8 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	}
 
 	private TypeMappedAnnotation(AnnotationTypeMapping mapping, @Nullable ClassLoader classLoader,
-			@Nullable Object source, @Nullable Object rootAnnotation,
-			BiFunction<Method, Object, Object> valueExtractor, int aggregateIndex,
-			boolean useMergedValues, @Nullable Predicate<String> attributeFilter,
+			@Nullable Object source, @Nullable Object rootAnnotation, ValueExtractor valueExtractor,
+			int aggregateIndex, boolean useMergedValues, @Nullable Predicate<String> attributeFilter,
 			int[] resolvedRootMirrors, int[] resolvedMirrors) {
 
 		this.classLoader = classLoader;
@@ -331,50 +327,20 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	}
 
 	@Override
+	@SuppressWarnings("unchecked")
 	protected A createSynthesized() {
+		if (getType().isInstance(this.rootAttributes) && !isSynthesizable()) {
+			return (A) this.rootAttributes;
+		}
 		return SynthesizedMergedAnnotationInvocationHandler.createProxy(this, getType());
 	}
 
-	@Override
-	public String toString() {
-		String string = this.string;
-		if (string == null) {
-			StringBuilder builder = new StringBuilder();
-			builder.append("@");
-			builder.append(getType().getName());
-			builder.append("(");
-			for (int i = 0; i < this.mapping.getAttributes().size(); i++) {
-				Method attribute = this.mapping.getAttributes().get(i);
-				builder.append(i == 0 ? "" : ", ");
-				builder.append(attribute.getName());
-				builder.append("=");
-				builder.append(toString(getValue(i, Object.class)));
-			}
-			builder.append(")");
-			string = builder.toString();
-			this.string = string;
+	private boolean isSynthesizable() {
+		// Already synthesized?
+		if (this.rootAttributes instanceof SynthesizedAnnotation) {
+			return false;
 		}
-		return string;
-	}
-
-	private Object toString(@Nullable Object value) {
-		if (value == null) {
-			return "";
-		}
-		if (value instanceof Class) {
-			return ((Class<?>) value).getName();
-		}
-		if (value.getClass().isArray()) {
-			StringBuilder builder = new StringBuilder();
-			builder.append("[");
-			for (int i = 0; i < Array.getLength(value); i++) {
-				builder.append(i == 0 ? "" : ", ");
-				builder.append(toString(Array.get(value, i)));
-			}
-			builder.append("]");
-			return builder.toString();
-		}
-		return String.valueOf(value);
+		return this.mapping.isSynthesizable();
 	}
 
 	@Override
@@ -417,25 +383,22 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 			}
 		}
 		if (!forMirrorResolution) {
-			attributeIndex = (mapping.getDistance() != 0 ?
-					this.resolvedMirrors :
-					this.resolvedRootMirrors)[attributeIndex];
+			attributeIndex =
+					(mapping.getDistance() != 0 ? this.resolvedMirrors : this.resolvedRootMirrors)[attributeIndex];
 		}
 		if (attributeIndex == -1) {
 			return null;
 		}
 		if (mapping.getDistance() == 0) {
 			Method attribute = mapping.getAttributes().get(attributeIndex);
-			Object result = this.valueExtractor.apply(attribute, this.rootAttributes);
-			return (result != null) ? result : attribute.getDefaultValue();
+			Object result = this.valueExtractor.extract(attribute, this.rootAttributes);
+			return (result != null ? result : attribute.getDefaultValue());
 		}
 		return getValueFromMetaAnnotation(attributeIndex, forMirrorResolution);
 	}
 
 	@Nullable
-	private Object getValueFromMetaAnnotation(int attributeIndex,
-			boolean forMirrorResolution) {
-
+	private Object getValueFromMetaAnnotation(int attributeIndex, boolean forMirrorResolution) {
 		Object value = null;
 		if (this.useMergedValues || forMirrorResolution) {
 			value = this.mapping.getMappedAnnotationValue(attributeIndex, forMirrorResolution);
@@ -469,7 +432,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 			value = ClassUtils.resolveClassName((String) value, getClassLoader());
 		}
 		else if (value instanceof Class[] && type == String[].class) {
-			Class<?>[] classes = (Class[]) value;
+			Class<?>[] classes = (Class<?>[]) value;
 			String[] names = new String[classes.length];
 			for (int i = 0; i < classes.length; i++) {
 				names[i] = classes[i].getName();
@@ -512,10 +475,9 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 			return adaptForAttribute(attribute, array);
 		}
 		if (attributeType.isAnnotation()) {
-			return adaptToMergedAnnotation(value,(Class<? extends Annotation>) attributeType);
+			return adaptToMergedAnnotation(value, (Class<? extends Annotation>) attributeType);
 		}
-		if (attributeType.isArray() && attributeType.getComponentType().isAnnotation() &&
-				value.getClass().isArray()) {
+		if (attributeType.isArray() && attributeType.getComponentType().isAnnotation()) {
 			MergedAnnotation<?>[] result = new MergedAnnotation<?>[Array.getLength(value)];
 			for (int i = 0; i < result.length; i++) {
 				result[i] = adaptToMergedAnnotation(Array.get(value, i),
@@ -562,7 +524,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 				mapping, null, this.source, value, getValueExtractor(value), this.aggregateIndex);
 	}
 
-	private BiFunction<Method, Object, Object> getValueExtractor(Object value) {
+	private ValueExtractor getValueExtractor(Object value) {
 		if (value instanceof Annotation) {
 			return ReflectionUtils::invokeMethod;
 		}
@@ -589,8 +551,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 
 	private int getAttributeIndex(String attributeName, boolean required) {
 		Assert.hasText(attributeName, "Attribute name must not be null");
-		int attributeIndex = (isFiltered(attributeName) ? -1 :
-				this.mapping.getAttributes().indexOf(attributeName));
+		int attributeIndex = (isFiltered(attributeName) ? -1 : this.mapping.getAttributes().indexOf(attributeName));
 		if (attributeIndex == -1 && required) {
 			throw new NoSuchElementException("No attribute named '" + attributeName +
 					"' present in merged annotation " + getType().getName());
@@ -612,7 +573,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 		}
 		if (this.source != null) {
 			if (this.source instanceof Class) {
-				return ((Class<?>) source).getClassLoader();
+				return ((Class<?>) this.source).getClassLoader();
 			}
 			if (this.source instanceof Member) {
 				((Member) this.source).getDeclaringClass().getClassLoader();
@@ -641,6 +602,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	@Nullable
 	static <A extends Annotation> TypeMappedAnnotation<A> createIfPossible(
 			AnnotationTypeMapping mapping, MergedAnnotation<?> annotation, IntrospectionFailureLogger logger) {
+
 		if (annotation instanceof TypeMappedAnnotation) {
 			TypeMappedAnnotation<?> typeMappedAnnotation = (TypeMappedAnnotation<?>) annotation;
 			return createIfPossible(mapping, typeMappedAnnotation.source,
@@ -664,17 +626,14 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 	@Nullable
 	private static <A extends Annotation> TypeMappedAnnotation<A> createIfPossible(
 			AnnotationTypeMapping mapping, @Nullable Object source, @Nullable Object rootAttribute,
-			BiFunction<Method, Object, Object> valueExtractor,
-			int aggregateIndex, IntrospectionFailureLogger logger) {
+			ValueExtractor valueExtractor, int aggregateIndex, IntrospectionFailureLogger logger) {
 
 		try {
 			return new TypeMappedAnnotation<>(mapping, null, source, rootAttribute,
 					valueExtractor, aggregateIndex);
 		}
 		catch (Exception ex) {
-			if (ex instanceof AnnotationConfigurationException) {
-				throw (AnnotationConfigurationException) ex;
-			}
+			AnnotationUtils.rethrowAnnotationConfigurationException(ex);
 			if (logger.isEnabled()) {
 				String type = mapping.getAnnotationType().getName();
 				String item = (mapping.getDistance() == 0 ? "annotation " + type :
@@ -687,7 +646,7 @@ final class TypeMappedAnnotation<A extends Annotation> extends AbstractMergedAnn
 
 	@SuppressWarnings("unchecked")
 	@Nullable
-	private static Object extractFromMap(Method attribute, @Nullable Object map) {
+	static Object extractFromMap(Method attribute, @Nullable Object map) {
 		return (map != null ? ((Map<String, ?>) map).get(attribute.getName()) : null);
 	}
 
