@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2018 the original author or authors.
+ * Copyright 2002-2020 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ package org.springframework.format.support;
 import org.springframework.core.convert.support.DefaultConversionService;
 import org.springframework.format.FormatterRegistry;
 import org.springframework.format.datetime.DateFormatterRegistrar;
-import org.springframework.format.datetime.joda.JodaTimeFormatterRegistrar;
 import org.springframework.format.datetime.standard.DateTimeFormatterRegistrar;
 import org.springframework.format.number.NumberFormatAnnotationFormatterFactory;
 import org.springframework.format.number.money.CurrencyUnitFormatter;
@@ -38,8 +37,8 @@ import org.springframework.util.StringValueResolver;
  * as {@code DefaultConversionService} exposes its own
  * {@link DefaultConversionService#addDefaultConverters addDefaultConverters} method.
  *
- * <p>Automatically registers formatters for JSR-354 Money & Currency, JSR-310 Date-Time
- * and/or Joda-Time, depending on the presence of the corresponding API on the classpath.
+ * <p>Automatically registers formatters for JSR-354 Money &amp; Currency, JSR-310 Date-Time
+ * and/or Joda-Time 2.x, depending on the presence of the corresponding API on the classpath.
  *
  * @author Chris Beams
  * @author Juergen Hoeller
@@ -54,10 +53,9 @@ public class DefaultFormattingConversionService extends FormattingConversionServ
 	static {
 		ClassLoader classLoader = DefaultFormattingConversionService.class.getClassLoader();
 		jsr354Present = ClassUtils.isPresent("javax.money.MonetaryAmount", classLoader);
-		jodaTimePresent = ClassUtils.isPresent("org.joda.time.LocalDate", classLoader);
+		jodaTimePresent = ClassUtils.isPresent("org.joda.time.YearMonth", classLoader);
 	}
 
-
 	/**
 	 * Create a new {@code DefaultFormattingConversionService} with the set of
 	 * {@linkplain DefaultConversionService#addDefaultConverters default converters} and
@@ -102,10 +100,11 @@ public class DefaultFormattingConversionService extends FormattingConversionServ
 
 	/**
 	 * Add formatters appropriate for most environments: including number formatters,
-	 * JSR-354 Money & Currency formatters, JSR-310 Date-Time and/or Joda-Time formatters,
+	 * JSR-354 Money &amp; Currency formatters, JSR-310 Date-Time and/or Joda-Time formatters,
 	 * depending on the presence of the corresponding API on the classpath.
 	 * @param formatterRegistry the service to register default formatters with
 	 */
+	@SuppressWarnings("deprecation")
 	public static void addDefaultFormatters(FormatterRegistry formatterRegistry) {
 		// Default handling of number values
 		formatterRegistry.addFormatterForFieldAnnotation(new NumberFormatAnnotationFormatterFactory());
@@ -124,7 +123,7 @@ public class DefaultFormattingConversionService extends FormattingConversionServ
 
 		if (jodaTimePresent) {
 			// handles Joda-specific types as well as Date, Calendar, Long
-			new JodaTimeFormatterRegistrar().registerFormatters(formatterRegistry);
+			new org.springframework.format.datetime.joda.JodaTimeFormatterRegistrar().registerFormatters(formatterRegistry);
 		}
 		else {
 			// regular DateFormat-based Date, Calendar, Long converters
